
/*******************************************************************************
* Copyright (c) 2017 Olivier Langella <olivier.langella@u-psud.fr>.
*
* This file is part of XTPcpp.
*
*     XTPcpp is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     XTPcpp is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with XTPcpp.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <olivier.langella@u-psud.fr> - initial API and implementation
******************************************************************************/

#include "peptidelistwindow.h"

#include "ui_peptide_view.h"


PeptideListWindow::PeptideListWindow(QWidget *parent):
    QMainWindow(parent),
    ui(new Ui::PeptideView)
{
    ui->setupUi(this);

#if QT_VERSION >= 0x050000
    // Qt5 code
    /*
    connect(&workerThread, &QThread::finished, worker, &QObject::deleteLater);
    connect(this, &PtSpectrumViewer::operateMsDataFile, worker, &PwizLoaderThread::doMsDataFileLoad);
    connect(worker, &PwizLoaderThread::msDataReady, this, &PtSpectrumViewer::handleMsDataFile);
    */
#else
// Qt4 code
    /*
        connect(&workerThread, SIGNAL(finished()), worker, SLOT(deleteLater()));
        connect(this, SIGNAL(operateMsDataFile(QString)), worker,SLOT(doMsDataFileLoad(QString)));
        connect(worker, SIGNAL(msDataReady(pwiz::msdata::MSDataFile *)), this, SLOT(handleMsDataFile(pwiz::msdata::MSDataFile *)));
        */
#endif
    /*
    */

    _peptide_table_model_p = new PeptideTableModel(0);


     _p_proxy_model = new PeptideTableProxyModel(this, _peptide_table_model_p);
    _p_proxy_model->setSourceModel(_peptide_table_model_p);
    _p_proxy_model->setDynamicSortFilter(true);
    ui->tableView->setModel( _p_proxy_model );
    ui->tableView->setSortingEnabled(true);
    
    connect(ui->tableView, SIGNAL(clicked(const QModelIndex &)), _p_proxy_model, SLOT(onTableClicked(const QModelIndex &)));
}

PeptideListWindow::~PeptideListWindow()
{
    //if (_p_ms_data_file != nullptr) delete _p_ms_data_file;
    delete ui;

}


void PeptideListWindow::setProteinMatch(Project * p_project, ProteinMatch * p_protein_match) {
  if (_p_protein_match != p_protein_match) {
        _p_protein_match = p_protein_match;
        _peptide_table_model_p->setProteinMatch(p_project, p_protein_match);
        _p_proxy_model->setSourceModel(_peptide_table_model_p);
        ui->proteinLabel->setText(p_protein_match->getProteinXtpSp().get()->getDescription());
    }

}

